@@ -2,7 +2,7 @@
  * File  : imageeffect_solarize.cpp
  * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
  * Date  : 2004-02-14
- * Description : a Digikam image plugin for to solarize
+ * Description : a digiKam image plugin for to solarize
  *               an image.
  *
  * Copyright 2004 by Renchi Raju
@@ -48,14 +48,14 @@
 // Local includes.
 
 #include "version.h"
+#include "bannerwidget.h"
 #include "imageeffect_solarize.h"
 
-
 namespace DigikamSolarizeImagesPlugin
 {
 
 ImageEffect_Solarize::ImageEffect_Solarize(QWidget* parent)
-                    : KDialogBase(Plain, i18n("Solarize Image"),
+                    : KDialogBase(Plain, i18n("Solarize Photograph"),
                                   Help|Ok|Cancel, Ok,
                                   parent, 0, true, true),
                       m_parent(parent)
@@ -63,7 +63,7 @@ ImageEffect_Solarize::ImageEffect_Solarize(QWidget* parent)
     // About data and help button.
 
     KAboutData* about = new KAboutData("digikamimageplugins",
-                                       I18N_NOOP("Solarize Image"),
+                                       I18N_NOOP("Solarize a Photograph"),
                                        digikamimageplugins_version,
                                        I18N_NOOP("A solarize image plugin for digiKam."),
                                        KAboutData::License_GPL,
@@ -77,72 +77,55 @@ ImageEffect_Solarize::ImageEffect_Solarize(QWidget* parent)
     m_helpButton = actionButton( Help );
     KHelpMenu* helpMenu = new KHelpMenu(this, about, false);
     helpMenu->menu()->removeItemAt(0);
-    helpMenu->menu()->insertItem(i18n("Solarize Image Handbook"), this, SLOT(slotHelp()), 0, -1, 0);
+    helpMenu->menu()->insertItem(i18n("Plugin Handbook"), this, SLOT(slotHelp()), 0, -1, 0);
     m_helpButton->setPopup( helpMenu->menu() );
 
     // -------------------------------------------------------------
 
     QVBoxLayout *topLayout = new QVBoxLayout( plainPage(), 0, spacingHint());
 
-    QFrame *headerFrame = new QFrame( plainPage() );
-    headerFrame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
-    QHBoxLayout* layout = new QHBoxLayout( headerFrame );
-    layout->setMargin( 2 ); // to make sure the frame gets displayed
-    layout->setSpacing( 0 );
-    QLabel *pixmapLabelLeft = new QLabel( headerFrame, "pixmapLabelLeft" );
-    pixmapLabelLeft->setScaledContents( false );
-    layout->addWidget( pixmapLabelLeft );
-    QLabel *labelTitle = new QLabel( i18n("Solarize Image"), headerFrame, "labelTitle" );
-    layout->addWidget( labelTitle );
-    layout->setStretchFactor( labelTitle, 1 );
+    QFrame *headerFrame = new DigikamImagePlugins::BannerWidget(plainPage(), i18n("Solarize Photograph"));    
     topLayout->addWidget(headerFrame);
 
-    QString directory;
-    KGlobal::dirs()->addResourceType("digikamimageplugins_banner_left", KGlobal::dirs()->kde_default("data") +
-                                                                        "digikamimageplugins/data");
-    directory = KGlobal::dirs()->findResourceDir("digikamimageplugins_banner_left",
-                                                 "digikamimageplugins_banner_left.png");
-
-    pixmapLabelLeft->setPaletteBackgroundColor( QColor(201, 208, 255) );
-    pixmapLabelLeft->setPixmap( QPixmap( directory + "digikamimageplugins_banner_left.png" ) );
-    labelTitle->setPaletteBackgroundColor( QColor(201, 208, 255) );
-
     // -------------------------------------------------------------
 
-    QVGroupBox *gbox = new QVGroupBox(i18n("Preview"), plainPage());
-    QFrame *frame = new QFrame(gbox);
+    QFrame *frame = new QFrame(plainPage());
     frame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
     QVBoxLayout* l = new QVBoxLayout(frame, 5, 0);
-    m_previewWidget = new Digikam::ImageWidget(480, 320,frame);
-    l->addWidget(m_previewWidget, 0, Qt::AlignCenter);
-    topLayout->addWidget(gbox);
+    m_previewWidget = new Digikam::ImageWidget(480, 320, frame);
+    QWhatsThis::add( m_previewWidget, i18n("<p>This is the solarize effect preview"));
+    l->addWidget(m_previewWidget, 0);
+    topLayout->addWidget(frame, 10);
 
+    // -------------------------------------------------------------
+                
     QHBoxLayout *hlay = new QHBoxLayout(topLayout);
     QLabel *label = new QLabel(i18n("Intensity:"), plainPage());
     m_numInput = new KDoubleNumInput(plainPage());
     m_numInput->setPrecision(1);
     m_numInput->setRange(0.0, 100.0, 0.1, true);
-    hlay->addWidget(label,1);
-    hlay->addWidget(m_numInput,5);
+    hlay->addWidget(label, 1);
+    hlay->addWidget(m_numInput, 5);
 
-    adjustSize();
-    disableResize();    
+    resize(configDialogSize("Solarize Tool Dialog"));      
     
     // -------------------------------------------------------------
 
     connect(m_numInput, SIGNAL(valueChanged (double)),
-            SLOT(slotEffect()));
+            this, SLOT(slotEffect()));
+            
+    connect(m_previewWidget, SIGNAL(signalResized()),
+            this, SLOT(slotEffect()));               
 }
 
 ImageEffect_Solarize::~ImageEffect_Solarize()
 {
-
+    saveDialogSize("Solarize Tool Dialog");
 }
 
 void ImageEffect_Solarize::slotHelp()
 {
-    KApplication::kApplication()->invokeHelp("solarizeimage",
-                                             "digikamimageplugins");
+    KApplication::kApplication()->invokeHelp("solarizeimage", "digikamimageplugins");
 }
 
 void ImageEffect_Solarize::closeEvent(QCloseEvent *e)
@@ -155,8 +138,7 @@ void ImageEffect_Solarize::closeEvent(QCloseEvent *e)
 
 void ImageEffect_Solarize::slotEffect()
 {
-    Digikam::ImageIface* iface =
-        m_previewWidget->imageIface();
+    Digikam::ImageIface* iface = m_previewWidget->imageIface();
 
     uint * data = iface->getPreviewData();
     int w       = iface->previewWidth();
@@ -165,23 +147,20 @@ void ImageEffect_Solarize::slotEffect()
     double factor = m_numInput->value();
 
     solarize(factor, data, w, h);
-
+    
     iface->putPreviewData(data);
-
     delete [] data;
-
     m_previewWidget->update();
 }
 
 void ImageEffect_Solarize::slotOk()
 {
-    m_parent->setCursor( KCursor::waitCursor() );
-    Digikam::ImageIface* iface =
-        m_previewWidget->imageIface();
+    kapp->setOverrideCursor( KCursor::waitCursor() );
+    Digikam::ImageIface iface(0, 0);
 
-    uint* data  = iface->getOriginalData();
-    int w       = iface->originalWidth();
-    int h       = iface->originalHeight();
+    uint* data  = iface.getOriginalData();
+    int w       = iface.originalWidth();
+    int h       = iface.originalHeight();
 
     if (data) 
        {
@@ -189,30 +168,30 @@ void ImageEffect_Solarize::slotOk()
 
        solarize(factor, data, w, h);
 
-       iface->putOriginalData(data);
-
+       iface.putOriginalData(i18n("Solarize"), data);
        delete [] data;
        }
 
-    m_parent->setCursor( KCursor::arrowCursor() );
+    kapp->restoreOverrideCursor();
     accept();
 }
 
 void ImageEffect_Solarize::solarize(double factor, uint *data, int w, int h)
 {
-    uint *ptr  = data;
-    uint a,r,g,b;
+    uint a, r, g, b;
+    Digikam::ImageFilters::imageData imagedata;
 
-    uint threshold = (uint)((100-factor)*(255+1)/100);
-    threshold = QMAX(1,threshold);
+    uint threshold = (uint)((100 - factor)*(255 + 1) / 100);
+    threshold = QMAX(1, threshold);
     bool stretch = true;
 
-    for (int x=0; x<w*h; x++) 
+    for (int x = 0; x < w*h; x++) 
        {
-       a = (*ptr >> 24) & 0xff;
-       r = (*ptr >> 16) & 0xff;
-       g = (*ptr >> 8 ) & 0xff;
-       b = (*ptr      ) & 0xff;
+       imagedata.raw = data[x];
+       r             = (uint)imagedata.channel.red;
+       g             = (uint)imagedata.channel.green;
+       b             = (uint)imagedata.channel.blue;
+       a             = (uint)imagedata.channel.alpha;
 
        if (stretch) 
           {
@@ -222,16 +201,16 @@ void ImageEffect_Solarize::solarize(double factor, uint *data, int w, int h)
           }
        else 
           {
-          if (r > threshold)
-             r = (255-r);
-          if (g > threshold)
-             g = (255-g);
-          if (b > threshold)
-             b = (255-b);
+          if (r > threshold) r = (255-r);
+          if (g > threshold) g = (255-g);
+          if (b > threshold) b = (255-b);
           }
 
-       *ptr = a << 24 | r << 16 | g << 8 | b;
-       ptr++;
+       imagedata.channel.red   = r;
+       imagedata.channel.green = g;
+       imagedata.channel.blue  = b;         
+       imagedata.channel.alpha = a;         
+       data[x] = imagedata.raw;
        }
 }
 
